/* **************************************************************************** */
/*        multi-mgMOS C implementation                                          */
/* **************************************************************************** */
#include <math.h>
#include "o8para.h"
#include "pplr_c.h"

#include <R.h> 
#include <Rdefines.h>
#include <Rmath.h>
#include <Rinternals.h>

static pplrparam in_param;
void donlp2(void);


void initialparams()
{
	in_param.replicates = NULL;
	in_param.data_m = NULL;
	in_param.data_var = NULL;
	in_param.expr = NULL;
	in_param.var = NULL;
	in_param.outp = NULL;
	in_param.mu1 = NULL;
	in_param.mu1sq = NULL;
}

void allocatemem()
{
	in_param.expr = (double*)Calloc(sizeof(double)*in_param.chips, double);
	in_param.var = (double*)Calloc(sizeof(double)*in_param.chips, double);
	if (in_param.method > 0)
	{
		in_param.mu1 = (double*)Calloc(sizeof(double)*in_param.chips, double);
		in_param.mu1sq = (double*)Calloc(sizeof(double)*in_param.chips, double);
	}
}

  
void freemem()
{
	if (in_param.expr != NULL) Free(in_param.expr);
	if (in_param.var != NULL) Free(in_param.var);
	if (in_param.method > 0)
	{
		if (in_param.mu1 != NULL) Free(in_param.mu1);
		if (in_param.mu1sq != NULL) Free(in_param.mu1sq);
	}
}

void findeforc(double *x, int c, double *e_c, int *num_c)
{
	int i,j;
	
	j = 0;
	for (i=0; i<in_param.chips; i++)
	{
		if (in_param.replicates[i] == c)
		{
			e_c[j] = x[i];
			j++;
		}
	}
	*num_c = j;
}

double mean(double *x, int len)
{
	int i;
	double s=0.0;
	
	for (i=0; i<len; i++)
		s += x[i];
	return s/len;
}

double var(double *x, int len)
{
	int i;
	double m,s=0.0;
	
	m = mean(x,len);
	for (i=0; i<len; i++)
		s += (x[i]-m)*(x[i]-m);
		
	return s/(len-1);
}

double fmaxn(double *x, int n)
{
	int i;
	double m;
	
	m = x[0];
	for (i=1; i<n; i++)
		if (x[i]>m)
			m = x[i];
			
	return m;
}

void workout_comb0()
{
    #define  X extern
    #include "o8comm.h"
    #undef   X
    #include "o8cons.h"
	int i, j;
     
	for (i=0; i<in_param.genes; i++)
	{
		for (j=0; j<in_param.chips; j++)
		{
			in_param.expr[j] = in_param.data_m[j*in_param.genes+i];
			in_param.var[j] = in_param.data_var[j*in_param.genes+i];
		}
		in_param.cur_gene = i;
			
		donlp2();
			
		/*Rprintf("%d  %d %d %d\n", i+1, icf, icgf, (int)optite+11);*/
		if ((int)i%200 == 0)
			Rprintf(".");
	}
}

void workout_comb1()
{
    #define  X extern
    #include "o8comm.h"
    #undef   X
    #include "o8cons.h"
	int i, j, k,num_c;
    	double exp_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE], x_temp[MAX_NUM_COND];
    	double v[MAX_NUM_COND], temp, tempx, foptold, fopt, lamda10[MAX_NUM_COND];
    	double mu_temp[MAX_NUM_COND], lam_temp[MAX_NUM_COND];
	double mu1old[MAX_NUM_COND], lamda1[MAX_NUM_COND], lamda1old[MAX_NUM_COND];
	double alpha1, alpha2, beta2, diff_mu1[MAX_NUM_COND], diff_lamda[MAX_NUM_COND];
	double *sample_x=NULL, *logy=NULL, *w_param=NULL;
	double beta1[MAX_NUM_CHIP], f_lamda[MAX_NUM_CHIP];
	double f_lamda_c[MAX_NUM_REPLICATE];
	
        sample_x = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	logy = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	w_param = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	
	for (i=0; i<in_param.genes; i++)
	{
		for (j=0; j<in_param.chips; j++)
		{
			in_param.expr[j] = in_param.data_m[j*in_param.genes+i];
			in_param.var[j] = in_param.data_var[j*in_param.genes+i];
		}
		in_param.cur_gene = i;
    		
		for (j=1; j<=in_param.conds; j++)
		{
			findeforc(in_param.expr, j, exp_c, &num_c);
			findeforc(in_param.var, j, var_c, &num_c);
			
			mu_temp[j-1] = mean(exp_c, num_c);
			temp = var(exp_c, num_c);
			lamda10[j-1] = 1/temp;
			tempx = 0.0;
			for (k=0; k<num_c; k++)
			{
				v[k] = fmax2(1.0e-4, temp-var_c[k]);
				if (v[k]>tempx)
					tempx = v[k];
			}
			lam_temp[j-1] = 1/mean(v, num_c);
		}
			
		in_param.mu0 = mean(mu_temp, in_param.conds);
		in_param.eta0 = 1/var(mu_temp, in_param.conds);
		if (tempx<=1.0e-3)
		{
			in_param.alpha0 = 10.0;
			in_param.beta0 = 1.0;
		}
		else
		{
			temp = mean(lam_temp, in_param.conds);
			tempx = var(lam_temp, in_param.conds);
			in_param.beta0 = temp/tempx;
			in_param.alpha0 = temp*temp/tempx;
		}
		
		foptold = POSI_INF;
		fopt = 1.0e18;
		while (foptold-fopt > in_param.eps)
		{
			for (j=0; j<in_param.conds; j++)
			{
				in_param.mu1[j] = mu_temp[j];
				lamda1[j] = lamda10[j];
				mu1old[j] = POSI_INF;
				lamda1old[j] = POSI_INF;
				diff_mu1[j] = fabs(in_param.mu1[j]-mu1old[j]);
				diff_lamda[j] = fabs(lamda1[j]-lamda1old[j]);
			}
			
			alpha1 = 3.0/2.0;
			alpha2 = in_param.alpha0;
			beta2 = in_param.beta0;
			
			GetRNGstate();
			for (j=0; j<in_param.sample_num; j++)
				sample_x[j] = rgamma(alpha2, 1.0/beta2);
			PutRNGstate();
			
			while (fmaxn(diff_mu1, in_param.conds)>in_param.eps || fmaxn(diff_lamda, in_param.conds)>in_param.eps)
			{
				for (j=0; j<in_param.conds; j++)
					in_param.mu1sq[j] = in_param.mu1[j]*in_param.mu1[j]+1.0/lamda1[j];
				for (k=0; k<in_param.sample_num; k++)
					logy[k] = 0.0;
				for (j=0; j<in_param.chips; j++)
				{
					beta1[j] = (in_param.expr[j]*in_param.expr[j]-2.0*in_param.mu1[in_param.replicates[j]-1]*in_param.expr[j]+in_param.mu1sq[in_param.replicates[j]-1])/2.0;
					for (k=0; k<in_param.sample_num; k++)
						 logy[k] += dgamma(1.0/(1.0/sample_x[k]+in_param.var[j]), alpha1, 1.0/beta1[j], TRUE);
					f_lamda[j] = 0.0;
				}
				tempx = fmaxn(logy, in_param.sample_num);
				temp = 0.0;
				for (k=0; k<in_param.sample_num; k++)
					temp += exp(logy[k]-tempx);
				in_param.lamda_m = 0.0;
				for (k=0; k<in_param.sample_num; k++)
				{
					w_param[k] = exp(logy[k]-tempx-log(temp));
					in_param.lamda_m += w_param[k]*sample_x[k];
					for (j=0; j<in_param.chips; j++)
						f_lamda[j] += w_param[k]/(1.0/sample_x[k]+in_param.var[j]);
				}
				
				for (j=0; j<in_param.conds; j++)
				{
					lamda1old[j] = lamda1[j];
					mu1old[j] = in_param.mu1[j];
					findeforc(f_lamda, j+1, f_lamda_c, &num_c);
					findeforc(in_param.expr, j+1, exp_c, &num_c);
					
					lamda1[j] = in_param.eta0;
					for (k=0; k<num_c; k++)
						lamda1[j] += f_lamda_c[k];
					in_param.mu1[j] = in_param.eta0*in_param.mu0/lamda1[j];
					for (k=0; k<num_c; k++)
						in_param.mu1[j] += f_lamda_c[k]*exp_c[k]/lamda1[j];
					
					diff_mu1[j] = fabs(in_param.mu1[j]-mu1old[j]);
					diff_lamda[j] = fabs(lamda1[j]-lamda1old[j]);
				}
			}
			in_param.loglamda = 0.0;
			for (k=0; k<in_param.sample_num; k++)
				in_param.loglamda += w_param[k]*log(sample_x[k]);
				
			foptold = fopt;
			
			donlp2();
			
			fopt = fx;
		
		}
		/*Rprintf("%d  %d %d %f %d %f\n", i+1, icf, icgf, fx, (int)optite+11,mm);*/
	
		for (j=0; j<in_param.conds; j++)
		{
			/* mean of expression level */
			in_param.outp[j*in_param.genes+i] = in_param.mu1[j];
			/* standard deviation of expression level */
			in_param.outp[(in_param.conds+j)*in_param.genes+i] = sqrt(1.0/lamda1[j]);
		}	
	
		if ((int)i%200 == 0)
			Rprintf(".");
	}
	Free(sample_x);
	Free(logy);
	Free(w_param);
}

void workout_comb2()
{
    #define  X extern
    #include "o8comm.h"
    #undef   X
    #include "o8cons.h"
	int i, j, k, q, num_c;
    	double exp_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE];
    	double temp, tempx, foptold, fopt;
	double mu1old, lamda1, lamda1old;
	double alpha1, alpha2, beta2;
	double *sample_x=NULL, *logy=NULL, *w_param=NULL;
	double beta1[MAX_NUM_CHIP], f_lamda[MAX_NUM_CHIP];
	
        sample_x = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	logy = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	w_param = (double*)Calloc(sizeof(double)*in_param.sample_num, double);
	
	for (i=0; i<in_param.genes; i++)
	{
		for (j=0; j<in_param.chips; j++)
		{
			in_param.expr[j] = in_param.data_m[j*in_param.genes+i];
			in_param.var[j] = in_param.data_var[j*in_param.genes+i];
		}
		in_param.cur_gene = i;
		
		for (j=0; j<in_param.conds; j++)
		{
			findeforc(in_param.expr, j+1, exp_c, &num_c);
			findeforc(in_param.var, j+1, var_c, &num_c);
			
			in_param.eta0 = 10.0;
			in_param.mu0 = mean(exp_c, num_c);
			in_param.alpha0 = 1.0;
			in_param.beta0 = 1.0;
			
			foptold = POSI_INF;
			fopt = 1.0e18;
			
			while ((foptold-fopt) > in_param.eps)
			{
				in_param.mu1[0] = mean(exp_c, num_c);
				lamda1 = 1.0;
				mu1old = POSI_INF;
				lamda1old = POSI_INF;
				
				alpha1 = 3.0/2.0;
				alpha2 = in_param.alpha0;
				beta2 = in_param.beta0;
				
				GetRNGstate();
				for (k=0; k<in_param.sample_num; k++)
					sample_x[k] = rgamma(alpha2, 1.0/beta2);
				PutRNGstate();
				
				while (fabs(in_param.mu1[0]-mu1old)>in_param.eps || fabs(lamda1-lamda1old)>in_param.eps)
				{
					in_param.mu1sq[0] = in_param.mu1[0]*in_param.mu1[0]+1.0/lamda1;
					for (k=0; k<in_param.sample_num; k++)
						logy[k] = 0.0;
					for (q=0; q<num_c; q++)
					{
						beta1[q] = (exp_c[q]*exp_c[q]-2.0*in_param.mu1[0]*exp_c[q]+in_param.mu1sq[0])/2.0;
						for (k=0; k<in_param.sample_num; k++)
							logy[k] += dgamma(1.0/(1.0/sample_x[k]+var_c[q]), alpha1, 1.0/beta1[q], TRUE);
						f_lamda[q] = 0.0;
					}
					tempx = fmaxn(logy, in_param.sample_num);
					temp = 0.0;
					for (k=0; k<in_param.sample_num; k++)
						temp += exp(logy[k]-tempx);
					in_param.lamda_m = 0.0;
					for (k=0; k<in_param.sample_num; k++)
					{
						w_param[k] = exp(logy[k]-tempx-log(temp));
						in_param.lamda_m += w_param[k]*sample_x[k];
						for (q=0; q<num_c; q++)
							f_lamda[q] += w_param[q]/(1.0/sample_x[k]+var_c[q]);
					}
					lamda1old = lamda1;
					mu1old = in_param.mu1[0];
					lamda1 = in_param.eta0;
					for (q=0; q<num_c; q++)
						lamda1 += f_lamda[q];
					in_param.mu1[0] = in_param.eta0*in_param.mu0/lamda1;
					for (q=0; q<num_c; q++)
						in_param.mu1[0] += f_lamda[q]*exp_c[q]/lamda1;
				}
				in_param.loglamda = 0.0;
				for (k=0; k<in_param.sample_num; k++)
					in_param.loglamda += w_param[k]*log(sample_x[k]);
				
				foptold = fopt;
				
				donlp2();
				fopt = fx;
			}
			/*Rprintf("%d %d %d %d %f %d\n", i+1, j+1, icf, icgf, fopt, (int)optite+11);*/
			/* mean of expression level */
			in_param.outp[j*in_param.genes+i] = in_param.mu1[0];
			/* standard deviation of expression level */
			in_param.outp[(in_param.conds+j)*in_param.genes+i] = sqrt(1.0/lamda1);
		}
		if ((int)i%200 == 0)
			Rprintf(".");
	}
	Free(sample_x);
	Free(logy);
	Free(w_param);
}

SEXP bcomb_c(SEXP Mmat, SEXP Stdmat, SEXP rep, SEXP method, SEXP conds, SEXP nsample, SEXP eps)
{
	SEXP dim=NULL;	
	SEXP res=NULL;

	initialparams();
	
	PROTECT(dim = getAttrib(Mmat, R_DimSymbol));
	in_param.genes = INTEGER(dim)[0];
	in_param.chips = INTEGER(dim)[1];
	
	in_param.conds = INTEGER_POINTER(AS_INTEGER(conds))[0];
	in_param.method = INTEGER_POINTER(AS_INTEGER(method))[0];
	in_param.eps = NUMERIC_POINTER(AS_NUMERIC(eps))[0];
	
	in_param.data_m = NUMERIC_POINTER(AS_NUMERIC(Mmat));
	in_param.data_var = NUMERIC_POINTER(AS_NUMERIC(Stdmat));
	
	in_param.replicates = INTEGER_POINTER(AS_INTEGER(rep));
	in_param.sample_num = INTEGER_POINTER(AS_INTEGER(nsample))[0];
	
	allocatemem();
	PROTECT(res = allocMatrix(REALSXP, in_param.genes, 2*in_param.conds));
	in_param.outp = NUMERIC_POINTER(AS_NUMERIC(res));
	
	Rprintf("Combined signals are computing ");
	if (in_param.method == 0)
		workout_comb0();   /* non-informative prior, shared sig^2 */
	else if (in_param.method == 1)
		workout_comb1();   /* conjugate prior, shared sig^2 */
	else
		workout_comb2();   /* conjugate prior, sig^2 is not shared */
	
	freemem();
	Rprintf("Done.\n");
	UNPROTECT(2);
	
	return res;
}

/* **************************************************************************** */
/*                              donlp2-intv size initialization                           */
/* **************************************************************************** */
void user_init_size(void) {
    #define  X extern
    #include "o8comm.h"
    #include "o8fint.h"
    #include "o8cons.h"
    #undef   X


    /* problem dimension n = dim(donlp2_x), nlin=number of linear constraints
       nonlin = number of nonlinear constraints  */
      
    if (in_param.method == 0)
    {
    	n = in_param.conds+3;
    	nstep = 20;
    }
    else if (in_param.method == 1)
    {
    	n = 4;
        nstep = 20;
    }
    else
    {
    	n = 4;
	nstep = 20;
    }
    
    nlin   =  0;
    nonlin =  0;
    iterma = 4000;
}

/* **************************************************************************** */
/*                              donlp2 standard setup                           */
/* **************************************************************************** */
void user_init(void) {
    #define  X extern
    #include "o8comm.h"
    #include "o8fint.h"
    #include "o8cons.h"
    #undef   X
    
    static IINTEGER i,j, num_c;
    double exp_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE], x_temp[MAX_NUM_COND];
    double v[MAX_NUM_COND];
    double mu_temp[MAX_NUM_COND], lam_temp[MAX_NUM_COND];

    silent = TRUE;
/*    intakt = TRUE;*/
    
    big = 1.e20;
    if (in_param.method==0)
    {
	/* initialise the parameters */
	for (j=1; j<=in_param.conds; j++)
	{
		findeforc(in_param.expr, j, exp_c, &num_c);
		donlp2_x[j] = mean(exp_c, num_c);
		x_temp[j-1] = donlp2_x[j];
		v[j-1] = var(exp_c, num_c);
	}
	donlp2_x[in_param.conds+1] = mean(v, in_param.conds);
	donlp2_x[in_param.conds+2] = mean(x_temp, in_param.conds);
	donlp2_x[in_param.conds+3] = var(x_temp, in_param.conds);
	
	for (i=1; i<=in_param.conds+3; i++)
	{
		low[i] = LOWBOUND;
		up[i] = big;
	}
    }
    else if (in_param.method == 1)
    {
    	donlp2_x[1] = in_param.eta0;
    	donlp2_x[2] = in_param.mu0;
    	donlp2_x[3] = in_param.alpha0;
    	donlp2_x[4] = in_param.beta0;
	
	for (i=1; i<=4; i++)
	{
		low[i] = LOWBOUND;
		up[i] = big;
	}
	low[2] = -big;
    	
    }
    else
    {
    	donlp2_x[1] = in_param.eta0;
    	donlp2_x[2] = in_param.mu0;
    	donlp2_x[3] = in_param.alpha0;
    	donlp2_x[4] = in_param.beta0;
	
	for (i=1; i<=4; i++)
	{
		low[i] = LOWBOUND;
		up[i] = big;
	}
	low[2] = -big;
    }

    analyt = TRUE;
    epsdif = 1.e-16;  
    
    nreset = n;
    
    del0 = 1.0e0;
    tau0 = 1.0e1;
    tau  = 0.1e0;
       
    return;
}

/* **************************************************************************** */
/*                                 special setup                                */
/* **************************************************************************** */
void setup(void) {
    #define  X extern
    #include "o8comm.h"
    #undef   X
   
    return;
}

/* **************************************************************************** */
/*  the user may add additional computations using the computed solution here   */
/* **************************************************************************** */
void solchk(void) {
    #define  X extern
    #include "o8comm.h"
    #undef   X
    #include "o8cons.h"

    IINTEGER i, j, num_c;
    double sig_ret, mu_ret, tau_ret;
    double t1,t2;
    double expr_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE];
    
    if (in_param.method == 0)
    {
    	sig_ret = donlp2_x[in_param.conds+1];
	mu_ret = donlp2_x[in_param.conds+2];
	tau_ret = donlp2_x[in_param.conds+3];
	for (j=0; j<in_param.conds; j++)
	{
		findeforc(in_param.expr, j+1, expr_c, &num_c);
		findeforc(in_param.var, j+1, var_c, &num_c);
		t1 = mu_ret/tau_ret;
		t2 = 1/tau_ret;
		for (i=0; i<num_c; i++)
		{
			t1 += expr_c[i]/(sig_ret+var_c[i]);
			t2 += 1.0/(sig_ret+var_c[i]);
		}
		/* mean of expression level */
		in_param.outp[j*in_param.genes+in_param.cur_gene] = t1/t2;
		/* standard deviation of expression level */
		in_param.outp[(in_param.conds+j)*in_param.genes+in_param.cur_gene] = sqrt(1.0/t2);
	}
    }
    else if (in_param.method == 1)
    {
    	in_param.eta0 = donlp2_x[1];
    	in_param.mu0 = donlp2_x[2];
    	in_param.alpha0 = donlp2_x[3];
    	in_param.beta0 = donlp2_x[4];
    }
    else
    {
    	in_param.eta0 = donlp2_x[1];
    	in_param.mu0 = donlp2_x[2];
    	in_param.alpha0 = donlp2_x[3];
    	in_param.beta0 = donlp2_x[4];
    }

    return;
}

/* **************************************************************************** */
/*                               objective function                             */
/* **************************************************************************** */
void ef(DDOUBLE donlp2_x[],DDOUBLE *fx) {
    #define  X extern
    #include "o8fuco.h"
    #undef   X

	int i, j;
	
	*fx = 0.0;
	
	if (in_param.method==0)
	{
		double sig_x, mu_x, tau_x, t1, t2;
		double expr_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE];
		int num_c;
	 	
		sig_x = donlp2_x[in_param.conds+1];
		mu_x = donlp2_x[in_param.conds+2];
		tau_x = donlp2_x[in_param.conds+3];
		
		*fx = (in_param.conds-1.0)*log(tau_x)/2.0;
		t1 = 0.0;
		t2 = 0.0;
		for (j=0; j<in_param.conds; j++)
		{
			findeforc(in_param.expr, j+1, expr_c, &num_c);
			findeforc(in_param.var, j+1, var_c, &num_c);
			for (i=0; i<num_c; i++)
			{
				t1 += log(sig_x+var_c[i]);
				t2 += (expr_c[i]-donlp2_x[j+1])*(expr_c[i]-donlp2_x[j+1])
					/(2.0*(sig_x+var_c[i]));
			}
			*fx += (donlp2_x[j+1]-mu_x)*(donlp2_x[j+1]-mu_x)/(2.0*tau_x);
		}
		*fx += t1/2.0+t2;
	}
	else if (in_param.method == 1)
	{
		double eta_x, u_x, alpha_x, beta_x;
		
		eta_x = donlp2_x[1];
		u_x = donlp2_x[2];
		alpha_x = donlp2_x[3];
		beta_x = donlp2_x[4];
		
		*fx = in_param.conds*log(eta_x)/2.0+alpha_x*in_param.loglamda-beta_x*in_param.lamda_m+alpha_x*log(beta_x)-lgammafn(alpha_x);
		for (i=0; i<in_param.conds; i++)
			*fx -= eta_x*(in_param.mu1sq[i]-2.0*in_param.mu1[i]*u_x+u_x*u_x)/2.0;
		*fx = -*fx;
	}
	else
	{
		double eta_x, u_x, alpha_x, beta_x;
		
		eta_x = donlp2_x[1];
		u_x = donlp2_x[2];
		alpha_x = donlp2_x[3];
		beta_x = donlp2_x[4];
		
		*fx = log(eta_x)/2.0-eta_x*(in_param.mu1sq[0]-2.0*in_param.mu1[0]*u_x+u_x*u_x)/2.0
			+alpha_x*in_param.loglamda-beta_x*in_param.lamda_m+alpha_x*log(beta_x)-lgammafn(alpha_x);
		*fx = -*fx;
	}
	
    return;
}

/* **************************************************************************** */
/*                          gradient of objective function                      */
/* **************************************************************************** */
void egradf(DDOUBLE donlp2_x[],DDOUBLE gradf[]) {
    #define  X extern
    #include "o8fuco.h"
    #undef   X

	int i, j;
    
	if (in_param.method==0)
	{
		double sig_x, mu_x, tau_x;
		double expr_c[MAX_NUM_REPLICATE], var_c[MAX_NUM_REPLICATE];
		int num_c;
	
		sig_x = donlp2_x[in_param.conds+1];
		mu_x = donlp2_x[in_param.conds+2];
		tau_x = donlp2_x[in_param.conds+3];
		
		gradf[in_param.conds+1] = 0.0;
		gradf[in_param.conds+2] = 0.0;
		gradf[in_param.conds+3] = (in_param.conds-1.0)/(2.0*tau_x);
		for (j=0; j<in_param.conds; j++)
		{
			gradf[j+1] = (donlp2_x[j+1]-mu_x)/tau_x;
			findeforc(in_param.expr, j+1, expr_c, &num_c);
			findeforc(in_param.var, j+1, var_c, &num_c);
			for (i=0; i<num_c; i++)
			{
				gradf[j+1] -= (expr_c[i]-donlp2_x[j+1])/(sig_x+var_c[i]);
				gradf[in_param.conds+1] += 1.0/(2.0*(sig_x+var_c[i]))-
						(expr_c[i]-donlp2_x[j+1])*(expr_c[i]-donlp2_x[j+1])/(2.0*(sig_x+var_c[i])*(sig_x+var_c[i]));
			}
			gradf[in_param.conds+2] -= (donlp2_x[j+1]-mu_x)/tau_x;
			gradf[in_param.conds+3] -= (donlp2_x[j+1]-mu_x)*(donlp2_x[j+1]-mu_x)/(2.0*tau_x*tau_x);
		}
	}
	else if (in_param.method == 1)
	{
		double eta_x, u_x, alpha_x, beta_x;
		
		eta_x = donlp2_x[1];
		u_x = donlp2_x[2];
		alpha_x = donlp2_x[3];
		beta_x = donlp2_x[4];
		
		gradf[1] = -in_param.conds/(2.0*eta_x);
		gradf[2] = 0.0;
		for (i=0; i<in_param.conds; i++)
		{
			gradf[1] += (in_param.mu1sq[i]-2.0*in_param.mu1[i]*u_x+u_x*u_x)/2.0;
			gradf[2] -= (in_param.mu1[i]-u_x)*eta_x;
		}
		gradf[3] = -in_param.loglamda-log(beta_x)+digamma(alpha_x);
		gradf[4] = in_param.lamda_m-alpha_x/beta_x;
	}
	else
	{
		double eta_x, u_x, alpha_x, beta_x;
		
		eta_x = donlp2_x[1];
		u_x = donlp2_x[2];
		alpha_x = donlp2_x[3];
		beta_x = donlp2_x[4];
		
		gradf[1] = -1.0/(2.0*eta_x)+(in_param.mu1sq[0]-2.0*in_param.mu1[0]*u_x+u_x*u_x)/2.0;;
		gradf[2] = -(in_param.mu1[0]-u_x)*eta_x;
		gradf[3] = -in_param.loglamda-log(beta_x)+digamma(alpha_x);
		gradf[4] = in_param.lamda_m-alpha_x/beta_x;
	}
    return;
}

/* **************************************************************************** */
/*  no nonlinear constraints */
/* **************************************************************************** */
void econ(IINTEGER type, IINTEGER liste[], DDOUBLE donlp2_x[], DDOUBLE con[], 
              LLOGICAL err[]) {
    #define  X extern
    #include "o8fuco.h"
    #undef   X

    return;
}

/* **************************************************************************** */
/* **************************************************************************** */
void econgrad(IINTEGER liste[], IINTEGER shift ,  DDOUBLE donlp2_x[],
               DDOUBLE **grad) {
    #define  X extern
    #include "o8fuco.h"
    #undef   X
    return;
}


/* **************************************************************************** */
/*                        user functions (if bloc == TRUE)                      */
/* **************************************************************************** */
void eval_extern(IINTEGER mode) {
    #define  X extern
    #include "o8comm.h"
    #include "o8fint.h"
    #undef   X
    #include "o8cons.h"

    return;
}
